@@ -18,6 +18,7 @@ my %builtin_method = (
     size    => [0, TX_ENUMERABLE],
     join    => [1, TX_ENUMERABLE],
     reverse => [0, TX_ENUMERABLE],
+    sort    => [0, TX_ENUMERABLE],
 
     keys    => [0, TX_KV],
     values  => [0, TX_KV],
@@ -50,7 +51,7 @@ sub tx_methodcall {
         my($nargs, $klass) = @{$bm};
         if(@args != $nargs) {
             tx_error($st,
-                "Builtin method %s requres exactly %d argument(s), "
+                "Builtin method %s requires exactly %d argument(s), "
                 . "but supplied %d",
                 $method, $nargs, scalar @args);
             return undef;
