@@ -158,8 +158,13 @@ sub op_fetch_field_s {
 sub op_print {
     my $sv = $_[0]->{sa};
 
-    if ( Scalar::Util::blessed( $sv ) and $sv->isa('Text::Xslate::EscapedString') ) {
-        $_[0]->{ output } .= $sv;
+    if ( ref( $sv ) eq 'Text::Xslate::EscapedString' ) {
+        if(defined ${$sv}) {
+            $_[0]->{ output } .= ${$sv};
+        }
+        else {
+            tx_warn( $_[0], "Use of nil to print" );
+        }
     }
     elsif ( defined $sv ) {
         if ( $sv =~ /[&<>"']/ ) {
@@ -414,6 +419,7 @@ sub op_ge {
 
 
 sub op_macrocall {
+    my $lvars  = $_[0]->pc_arg;
     my $addr   = $_[0]->{sa}; # macro entry point
     my $cframe = tx_push_frame( $_[0] );
 
@@ -423,6 +429,15 @@ sub op_macrocall {
 
     $_[0]->{ output } = '';
 
+    if($lvars > 0) {
+        # copies lexical variables from the old frame to the new one
+        my $oframe = $_[0]->frame->[ $_[0]->current_frame - 1 ];
+        for(my $i = 0; $i < $lvars; $i++) {
+            my $real_ix = $i + TXframe_START_LVAR;
+            $cframe->[$real_ix] = $oframe->[$real_ix];
+        }
+    }
+
     my $i   = 0;
 
     $_[0]->pad( $_[0]->frame->[ $_[0]->current_frame ] );
