@@ -54,8 +54,6 @@ my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
 
-our $in_given;
-
 has symbol_table => ( # the global symbol table
     is  => 'ro',
     isa => 'HashRef',
@@ -136,6 +134,12 @@ has shortcut_table => (
 );
 sub _build_shortcut_table { \%shortcut_table }
 
+has in_given => (
+    is       => 'rw',
+    isa      => 'Bool',
+    init_arg => undef,
+);
+
 # attributes for error messages
 
 has near_token => (
@@ -145,24 +149,22 @@ has near_token => (
 );
 
 has file => (
-    is  => 'rw',
-    isa => 'Str',
-
+    is       => 'rw',
+    isa      => 'Str',
     required => 0,
 );
 
 has line => (
-    is  => 'rw',
-    isa => 'Int',
-
-    traits  => [qw(Counter)],
-    handles => {
-        line_inc => 'inc',
-    },
-
+    is       => 'rw',
+    isa      => 'Int',
     required => 0,
 );
 
+sub line_inc {
+    my($parser) = @_;
+    return $parser->line( $parser->line + 1);
+}
+
 sub symbol_class() { 'Text::Xslate::Symbol' }
 
 sub _trim {
@@ -176,14 +178,14 @@ sub _trim {
 
 # split templates by tags before tokanizing
 sub split :method {
-    my $self  = shift;
+    my $parser  = shift;
     local($_) = @_;
 
     my @tokens;
 
-    my $line_start    = $self->line_start;
-    my $tag_start     = $self->tag_start;
-    my $tag_end       = $self->tag_end;
+    my $line_start    = $parser->line_start;
+    my $tag_start     = $parser->tag_start;
+    my $tag_end       = $parser->tag_end;
 
     my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* $line_start ([^\n]* \n?) /xms;
     my $lex_tag_start = qr/\A $tag_start ($CHOMP_FLAGS?)/xms;
@@ -206,8 +208,8 @@ sub split :method {
                 }
             }
             else {
-                $self->near_token((split /\n/, $_)[0]);
-                $self->_error("Malformed templates");
+                $parser->near_token((split /\n/, $_)[0]);
+                $parser->_error("Malformed templates");
             }
         }
         # not $in_tag
@@ -235,12 +237,12 @@ sub split :method {
 }
 
 sub preprocess {
-    my $self = shift;
+    my $parser = shift;
 
-    my $tokens_ref = $self->split(@_);
+    my $tokens_ref = $parser->split(@_);
     my $code = '';
 
-    my $shortcut_table = $self->shortcut_table;
+    my $shortcut_table = $parser->shortcut_table;
     my $shortcut       = join('|', map{ quotemeta } keys %shortcut_table);
     my $shortcut_rx    = qr/\A ($shortcut)/xms;
 
@@ -300,7 +302,7 @@ sub preprocess {
             # noop, just a marker
         }
         else {
-            $self->_error("Oops: Unknown token: $s ($type)");
+            $parser->_error("Oops: Unknown token: $s ($type)");
         }
     }
     print STDOUT $code, "\n" if _DUMP_PROTO;
@@ -308,11 +310,11 @@ sub preprocess {
 }
 
 sub lex {
-    my($self) = @_;
+    my($parser) = @_;
 
-    local *_ = \$self->{input};
+    local *_ = \$parser->{input};
 
-    s{\G (\s) }{ $1 eq "\n" and $self->line_inc(); ""}xmsge;
+    s{\G (\s) }{ $1 eq "\n" and $parser->line_inc(); ""}xmsge;
 
     if(s/\A ($ID)//xmso){
         return [ name => $1 ];
@@ -330,7 +332,7 @@ sub lex {
         return [ string => $1 ];
     }
     elsif(s/\A (\S+)//xms) {
-        $self->_error("Oops: Unexpected lex symbol '$1'");
+        $parser->_error("Oops: Unexpected lex symbol '$1'");
     }
     else { # empty
         return undef;
@@ -341,14 +343,14 @@ sub parse {
     my($parser, $input, %args) = @_;
 
     $parser->file( $args{file} || '<input>' );
-    $parser->line( $args{line} || 0 );
-    $parser->near_token('(start)');
-    $parser->token(undef);
+    $parser->line( $args{line} || 1 );
     $parser->init_scope();
-
-    local $in_given = 0;
+    $parser->in_given(0);
 
     local $parser->{symbol_table} = { %{ $parser->symbol_table } };
+    local $parser->{near_token};
+    local $parser->{next_token};
+    local $parser->{token};
 
     $parser->input( $parser->preprocess($input) );
 
@@ -359,8 +361,6 @@ sub parse {
     if($parser->input ne '') {
         $parser->_error("Syntax error", $parser->token);
     }
-    $parser->near_token(undef);
-    $parser->next_token(undef);
 
     return $ast;
 }
@@ -398,6 +398,10 @@ sub _init_basic_symbols {
     $parser->symbol(';');
     $parser->symbol('(');
     $parser->symbol(')');
+    $parser->symbol('{');
+    $parser->symbol('}');
+    $parser->symbol('[');
+    $parser->symbol(']');
     $parser->symbol(',')  ->is_comma(1);
     $parser->symbol('=>') ->is_comma(1);
 
@@ -405,10 +409,10 @@ sub _init_basic_symbols {
     $parser->symbol('print')    ->set_std(\&std_command);
     $parser->symbol('print_raw')->set_std(\&std_command);
 
-    # common constants
-    $parser->define_constant(nil   => undef);
-    $parser->define_constant(true  => 1);
-    $parser->define_constant(false => 0);
+    # common literals
+    $parser->define_literal(nil   => undef);
+    $parser->define_literal(true  => 1);
+    $parser->define_literal(false => 0);
 
     return;
 }
@@ -425,7 +429,7 @@ sub init_basic_operators {
     $parser->infix('.', 256, \&led_dot);
     $parser->infix('[', 256, \&led_fetch);
 
-    $parser->prefix('(', 200, \&nud_paren);
+    $parser->prefix('(', 256, \&nud_paren);
 
     $parser->prefix('!', 200)->is_logical(1);
     $parser->prefix('+', 200);
@@ -481,7 +485,6 @@ sub init_symbols {
     my($parser) = @_;
 
     # syntax specific separators
-    $parser->symbol(']');
     $parser->symbol('}')->is_block_end(1); # block end
     $parser->symbol('->');
     $parser->symbol('else');
@@ -513,6 +516,10 @@ sub init_symbols {
     $parser->symbol('super')    ->set_std(\&std_marker);
     $parser->symbol('override') ->set_std(\&std_override);
 
+    # lexical variable stuff
+    $parser->symbol('constant')->set_nud(\&nud_constant);
+    $parser->symbol('my'      )->set_nud(\&nud_constant);
+
     return;
 }
 
@@ -602,7 +609,11 @@ sub advance {
         Carp::confess("Panic: Unexpected token: $value ($arity)");
     }
 
-    return $parser->token( $proto->clone( id => $value, arity => $arity, line => $parser->line + 1 ) );
+    return $parser->token( $proto->clone(
+        id    => $value,
+        arity => $arity,
+        line  => $parser->line,
+     ) );
 }
 
 sub expression {
@@ -625,23 +636,19 @@ sub expression {
 
 sub expression_list {
     my($parser) = @_;
-
-    my @args;
-
-    if($parser->token->is_value or $parser->token->is_comma) {
-        while(1) {
-            if($parser->token->is_value) {
-                push @args, $parser->expression(0);
-            }
-
-            if(!$parser->token->is_comma) {
-                last;
-            }
-
-            $parser->advance(); # comma
+    my @list;
+    while(1) {
+        if($parser->token->is_value) {
+            push @list, $parser->expression(0);
         }
+
+        if(!$parser->token->is_comma) {
+            last;
+        }
+
+        $parser->advance(); # comma
     }
-    return \@args;
+    return \@list;
 }
 
 sub led_infix {
@@ -693,13 +700,13 @@ sub assignment {
 sub led_ternary {
     my($parser, $symbol, $left) = @_;
 
-    my $cond = $symbol->clone(arity => 'ternary');
+    my $if = $symbol->clone(arity => 'if');
 
-    $cond->first($left);
-    $cond->second($parser->expression( $cond->lbp - 1 ));
+    $if->first($left);
+    $if->second([$parser->expression( $symbol->lbp - 1 )]);
     $parser->advance(":");
-    $cond->third($parser->expression( $cond->lbp - 1 ));
-    return $cond;
+    $if->third([$parser->expression( $symbol->lbp - 1 )]);
+    return $if;
 }
 
 sub is_valid_field {
@@ -791,7 +798,7 @@ sub prefix {
     return $symbol;
 }
 
-sub define_constant {
+sub define_literal{
     my($parser, $id, $value) = @_;
 
     my $symbol = $parser->symbol($id);
@@ -824,25 +831,27 @@ sub undefined_name {
 
 sub find { # find a name from all the scopes
     my($parser, $name) = @_;
+    my $s;
     foreach my $scope(reverse @{$parser->scope}){
-        my $o = $scope->{$name};
-        if(defined $o) {
-            return $o;
+        $s = $scope->{$name};
+        if(defined $s) {
+            return $s;
         }
     }
-    return $parser->symbol_table->{$name} || $parser->undefined_name($name);
+    $s = $parser->symbol_table->{$name};
+    return defined($s) ? $s : $parser->undefined_name($name);
 }
 
 sub reserve { # reserve a name to the scope
     my($parser, $symbol) = @_;
-    if($symbol->arity ne 'name' or $symbol->reserved) {
+    if($symbol->arity ne 'name' or $symbol->is_reserved) {
         return $symbol;
     }
 
     my $top = $parser->scope->[-1];
     my $t = $top->{$symbol->id};
     if($t) {
-        if($t->reserved) {
+        if($t->is_reserved) {
             return $symbol;
         }
         if($t->arity eq "name") {
@@ -850,7 +859,7 @@ sub reserve { # reserve a name to the scope
         }
     }
     $top->{$symbol->id} = $symbol;
-    $symbol->reserved(1);
+    $symbol->is_reserved(1);
     #$symbol->scope($top);
     return $symbol;
 }
@@ -861,12 +870,13 @@ sub define { # define a name to the scope
 
     my $t = $top->{$symbol->id};
     if(defined $t) {
-        $parser->_error($t->reserved ? "Already reserved: $t" : "Already defined: $t");
+        $parser->_error($t->is_reserved ? "Already is_reserved: $t" : "Already defined: $t");
     }
 
     $top->{$symbol->id} = $symbol;
 
-    $symbol->reserved(0);
+    $symbol->is_defined(1);
+    $symbol->is_reserved(0);
     $symbol->remove_nud();
     $symbol->remove_led();
     $symbol->remove_std();
@@ -935,12 +945,17 @@ sub statement { # process one or more statements
     my $expr = $parser->expression(0);
     $parser->finish_statement();
 
-    return $parser->symbol('print')->clone(
-        arity  => 'command',
-        first  => [$expr],
-        line   => $expr->line,
-    );
-    #return $expr;
+    if($expr->is_statement) {
+        # expressions can produce statements (e.g. assignment)
+        return $expr;
+    }
+    else {
+        return $parser->symbol('print')->clone(
+            arity  => 'command',
+            first  => [$expr],
+            line   => $expr->line,
+        );
+    }
 }
 
 sub statements { # process statements
@@ -1017,6 +1032,30 @@ sub nud_iterator {
     return $iterator;
 }
 
+sub nud_constant {
+    my($parser, $symbol) = @_;
+    my $t = $parser->token;
+
+    my $expect =  $symbol->id eq 'constant' ? 'name'
+                : $symbol->id eq 'my'       ? 'variable'
+                :  die "Oops: $symbol";
+
+    if($t->arity ne $expect) {
+        $parser->_unexpected("a $expect", $t);
+    }
+    $parser->define($t)->arity("name");
+
+    $parser->advance();
+    $parser->advance("=");
+
+    return $symbol->clone(
+        arity        => 'constant',
+        first        => $t,
+        second       => $parser->expression(0),
+        is_statement => 1,
+    );
+}
+
 sub std_block {
     my($parser, $symbol) = @_;
     $parser->new_scope();
@@ -1210,8 +1249,8 @@ sub std_if {
         $t = $parser->advance(); # "else"
 
         $if->third( $t->id eq "if"
-            ? $parser->statement()
-            : $parser->block());
+            ? [$parser->statement()]
+            :  $parser->block());
     }
     return $top_if;
 }
@@ -1222,7 +1261,7 @@ sub std_given {
     my $proc = $symbol->clone(arity => 'given');
     $proc->first( $parser->expression(0) );
 
-    local $in_given = 1;
+    local $parser->{in_given} = 1;
     $parser->pointy($proc);
 
     if(!(defined $proc->second && @{$proc->second})) { # if no vars given
@@ -1288,7 +1327,7 @@ sub std_given {
 sub std_when {
     my($parser, $symbol) = @_;
 
-    if(!$in_given) {
+    if(!$parser->in_given) {
         $parser->_error("You cannot use $symbol blocks outside given blocks");
     }
     my $proc = $symbol->clone(arity => 'when');
@@ -1367,7 +1406,9 @@ sub localize_vars {
     my($parser) = @_;
     if($parser->token->id eq "{") {
         $parser->advance();
+        $parser->new_scope();
         my $vars = $parser->expression_list();
+        $parser->pop_scope();
         $parser->advance("}");
         return $vars;
     }
@@ -1541,10 +1582,10 @@ sub iterator_peek_prev {
 
     # $~iterator.is_first ? nil : <prev>
     return $parser->symbol('?')->clone(
-        arity  => 'ternary',
+        arity  => 'if',
         first  => $is_first,
-        second => $nil,
-        third  => $prev,
+        second => [$nil],
+        third  => [$prev],
     );
 }
 
@@ -1563,18 +1604,18 @@ sub _unexpected {
 }
 
 sub _error {
-    my($self, $message, $near) = @_;
+    my($parser, $message, $near) = @_;
 
-    $near ||= $self->near_token;
+    $near ||= $parser->near_token || ";";
     if($near ne ";") {
-        $near = sprintf ' near %s (%s)', $near->id, $near->arity
+        $near = sprintf ', near %s (%s)', $near->id, $near->arity
             if ref($near);
     }
     else {
         $near = '';
     }
     Carp::croak(sprintf 'Xslate::Parser(%s:%d): %s%s while parsing templates',
-        $self->file, $self->line+1, $message, $near);
+        $parser->file, $parser->line, $message, $near);
 }
 
 no Any::Moose;
